Package: climniche
Type: Package
Title: Niche Climate Exposure
Version: 0.0.1
Authors@R: c(
    person("Bohao", "He", role = c("aut", "cre"), email = "bohao.he@polimi.it")
    )
Description: Assesses niche climate exposure by interpreting projected climate
    change relative to the climate conditions a species currently occupies.
    Using occurrence records, range data or thresholded SDM suitability maps,
    current environmental rasters and future projections, the package separates
    climate change amount, change in distance to the current niche centre,
    composition change and exceedance beyond an empirical niche boundary.
License: GPL (>= 3)
URL: https://github.com/Bohao0813/climniche,
        https://bohao0813.github.io/climniche/
BugReports: https://github.com/Bohao0813/climniche/issues
Encoding: UTF-8
Imports: grid, methods
Suggests: ggplot2, knitr, patchwork, raster, rmarkdown, terra
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-26 13:11:44 UTC; Bohao He
Author: Bohao He [aut, cre]
Maintainer: Bohao He <bohao.he@polimi.it>
Repository: CRAN
Date/Publication: 2026-05-29 12:20:02 UTC
Built: R 4.5.2; ; 2026-05-29 13:30:10 UTC; unix
