export(fit_climniche)
export(fit_climniche_raster)
export(fit_climniche_terra)
export(climniche_diagram_data)
export(climniche_showcase_data)
export(niche_metric)
export(niche_potential)
export(niche_radius)
export(plot_climniche_map)
export(plot_climniche_classes)
export(plot_climniche_exposure)
export(plot_climniche_class_summary)
export(plot_climniche_distribution)
export(plot_climniche_report)
export(plot_climniche_diagram)
export(plot_climniche_showcase)
export(plot_climniche_variable_contribution)
export(plot_variable_contribution)
export(climniche_table)
export(climniche_summary)
export(climniche_report)
export(write_climniche_report)
export(boundary_exceedance)
export(niche_percentile)
export(variable_contribution)
export(simulate_climniche)
importFrom(methods,is)
importFrom(methods,slot)
importFrom(methods,slotNames)
S3method(summary,climniche_fit)
S3method(plot,climniche_fit)
S3method(print,climniche_report)
