% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aliases.R
\name{plot_climniche_distribution}
\alias{plot_climniche_distribution}
\title{Plot a climniche metric distribution}
\usage{
plot_climniche_distribution(
  x,
  metric = c("niche_distance_change", "climate_change_amount",
    "outside_niche_exceedance", "composition_change"),
  scope = c("current", "all"),
  title = NULL
)
}
\arguments{
\item{x}{A fitted climniche object.}

\item{metric}{Metric to plot.}

\item{scope}{\code{"current"} for current occurrence/range cells or \code{"all"} for
all evaluated cells.}

\item{title}{Optional plot title.}
}
\value{
A ggplot object.
}
\description{
Plot a climniche metric distribution
}
