% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aliases.R
\name{plot_climniche_classes}
\alias{plot_climniche_classes}
\title{Plot climniche classes}
\usage{
plot_climniche_classes(
  x,
  occupied = NULL,
  occupied_only = FALSE,
  occupied_threshold = 0,
  title = NULL
)
}
\arguments{
\item{x}{A fitted climniche object with raster outputs.}

\item{occupied}{Optional current occurrence/range RasterLayer to overlay.}

\item{occupied_only}{If TRUE, mask the plotted classes to current occurrence
cells.}

\item{occupied_threshold}{Threshold used when \code{occupied} contains binary or
continuous values.}

\item{title}{Optional plot title. Use \code{FALSE} to suppress it.}
}
\value{
A ggplot object.
}
\description{
Plot climniche classes
}
