% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aliases.R
\name{plot_climniche_exposure}
\alias{plot_climniche_exposure}
\title{Plot the climniche exposure plane}
\usage{
plot_climniche_exposure(
  x,
  scope = c("current", "all"),
  max_points = 6000,
  seed = 1,
  title = NULL
)
}
\arguments{
\item{x}{A fitted climniche object.}

\item{scope}{\code{"current"} for current occurrence/range cells or \code{"all"} for
all evaluated cells.}

\item{max_points}{Maximum number of points to draw.}

\item{seed}{Random seed used when subsampling.}

\item{title}{Optional plot title.}
}
\value{
A ggplot object.
}
\description{
Plot the climniche exposure plane
}
