% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aliases.R
\name{plot_climniche_report}
\alias{plot_climniche_report}
\title{Plot a climniche report figure}
\usage{
plot_climniche_report(x, scope = c("current", "all"))
}
\arguments{
\item{x}{A fitted climniche object.}

\item{scope}{\code{"current"} for current occurrence/range cells or \code{"all"} for
all evaluated cells.}
}
\value{
A patchwork object when \code{patchwork} is installed, otherwise a named
list of ggplot objects.
}
\description{
Plot a climniche report figure
}
