% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{simulate_climniche}
\alias{simulate_climniche}
\title{Simulate a minimal climate niche change experiment}
\usage{
simulate_climniche(
  n = 2000,
  p = 2,
  seed = 1,
  rho = 0,
  prevalence = 0.3,
  shift = 0.4
)
}
\arguments{
\item{n}{Number of climate cells.}

\item{p}{Number of climate variables.}

\item{seed}{Random seed.}

\item{rho}{Pairwise correlation among simulated climate variables.}

\item{prevalence}{Proportion of background cells treated as true current
occurrence locations under the virtual niche.}

\item{shift}{Climate change amount imposed in the closer to niche and
farther from niche scenarios.}
}
\value{
A list with current, future_toward, future_away, occupied, center,
sensitivity and A.
}
\description{
Simulate a minimal climate niche change experiment
}
