% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.R
\name{climniche_summary}
\alias{climniche_summary}
\title{Summarise climniche results}
\usage{
climniche_summary(x, scope = c("current", "all"))
}
\arguments{
\item{x}{A fitted \code{climniche_fit} object.}

\item{scope}{\code{"current"} for current occurrence/range cells or \code{"all"} for
all evaluated cells.}
}
\value{
A one-row data frame with key report metrics.
}
\description{
Summarise climniche results
}
