% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aliases.R
\name{fit_climniche_terra}
\alias{fit_climniche_terra}
\title{Fit climniche to terra raster data}
\usage{
fit_climniche_terra(
  current,
  future,
  occupied = NULL,
  occupied_threshold = 0,
  domain = NULL,
  domain_threshold = 0,
  ...
)
}
\arguments{
\item{current}{terra SpatRaster of current environmental layers.}

\item{future}{terra SpatRaster of future environmental layers.}

\item{occupied}{Optional one layer SpatRaster with binary or continuous
occurrence, range, or SDM suitability values.}

\item{occupied_threshold}{Values greater than this threshold are treated as
current occurrence cells.}

\item{domain}{Optional one layer SpatRaster limiting cells where exposure is
analysed.}

\item{domain_threshold}{Values greater than this threshold define the domain.}

\item{...}{Additional arguments passed to \code{fit_climniche()}.}
}
\value{
An object of class \code{climniche_fit} with raster outputs.
}
\description{
Fit climniche to terra raster data
}
